use std::collections::HashMap;
use std::sync::{Arc, Mutex};

type EventCallback = Arc<dyn Fn() + Send + Sync>;

struct EventEmitter {
    _events: Mutex<HashMap<String, Vec<EventCallback>>>,
    _max_listeners: usize,
}

impl EventEmitter {
    pub fn new() -> Self {
        EventEmitter {
            _events: Mutex::new(HashMap::new()),
            _max_listeners: usize::MAX,
        }
    }

    /// 设置最大监听器数量
    pub fn set_max_listeners(&mut self, max_listeners: usize) {
        self._max_listeners = max_listeners;
    }

    /// 获取最大监听器数量
    pub fn get_max_listeners(&self) -> usize {
        self._max_listeners
    }

    /// 添加事件监听器
    pub fn add_listener(&self, event: &str, callback: EventCallback) {
        self.on(event, callback);
    }

    pub fn on(&self, event: &str, callback: EventCallback) {
        let mut events = self._events.lock().unwrap();
        let callbacks = events.entry(event.to_string()).or_insert(Vec::new());
        callbacks.push(callback);
    }

    /// 移除事件监听器
    pub fn remove_listener(&self, event: &str, callback: &EventCallback) {
        self.off(event, callback);
    }

    pub fn off(&self, event: &str, callback: &EventCallback) {
        let mut events = self._events.lock().unwrap();
        if let Some(callbacks) = events.get_mut(event) {
            callbacks.retain(|cb| !Arc::ptr_eq(cb, callback));
        }
    }

    /// 添加一次性事件监听器
    // pub fn once(&self, event: &str, callback: EventCallback) {
    //     let callback_wrapper = Arc::new(move || {
    //         self.off(event, &callback);
    //         (*callback)();
    //     });
    //     self.on(event, callback_wrapper);
    // }

    

    /// 触发事件
    pub fn emit(&self, event: &str) {
        let events = self._events.lock().unwrap();
        if let Some(callbacks) = events.get(event) {
            for callback in callbacks {
                let callback_clone = callback.clone();
                // Spawn a new thread to run each callback asynchronously
                std::thread::spawn(move || {
                    (*callback_clone)();
                });
            }
        }
    }

    /// 移除所有事件的所有监听器
    pub fn remove_all_listeners(&self, event: &str) {
        let mut events = self._events.lock().unwrap();
        events.remove(event);
    }

    pub fn prepend_listener(&self, event: &str, callback: EventCallback) {
        let mut events = self._events.lock().unwrap();
        let callbacks = events.entry(event.to_string()).or_insert(Vec::new());
        callbacks.insert(0, callback);
    }

    // pub fn prepend_once_listener(&self, event: &str, callback: EventCallback) {
    //     let callback_wrapper = Arc::new(move || {
    //         self.off(event, &callback);
    //         (*callback)();
    //     });
    //     self.prepend_listener(event, callback_wrapper);
    // }

    pub fn listeners(&self, event: &str) -> Vec<EventCallback> {
        let events = self._events.lock().unwrap();
        if let Some(callbacks) = events.get(event) {
            callbacks.clone()
        } else {
            Vec::new()
        }
    }

    /// 获取指定事件的监听器数量
    pub fn listener_count(&self, event: &str) -> usize {
        let events = self._events.lock().unwrap();
        if let Some(callbacks) = events.get(event) {
            callbacks.len()
        } else {
            0
        }
    }
}